unit Matrices;

interface
uses GL, GLU, SDL;

type M4 = array[0..3,0..3] of GLFloat;
     V3 = array[0..2] of GLFloat;
     V4 = array[0..3] of GLFloat;

type Matrix = class
published
   procedure translate(x : GLFloat; y : GLFloat; z : GLFloat);
   procedure rotate(angle : GLFloat; x : GLFloat; y : GLFloat; z : GLFloat);
   procedure setData(dataIn : M4);
   function getData : M4;
   procedure makeIdentity;
   procedure useThis;
private
   data : array[0..15] of GLFloat;
end;

function vec3(x:GLFloat;y:GLFloat;z:GLFloat) : V3;
function multiplyMatrixWithVec(matr : M4;vec : V3) : V3;

implementation
    procedure Matrix.translate(x : GLFloat; y : GLFloat; z : GLFloat);
    begin
        glMATRIXMODE(GL_MODELVIEW);
        glPushMatrix;
        glLoadMatrixf(@data);
        glTranslatef(x,y,z);
        glGetFloatv(GL_MODELVIEW_MATRIX, @data);
        glPopMatrix;
    end;

    procedure Matrix.rotate(angle : GLFloat; x : GLFloat; y : GLFloat; z : GLFloat);
    begin
        glMATRIXMODE(GL_MODELVIEW);
        glPushMatrix;
        glLoadMatrixf(@data);
        glRotatef(angle,x,y,z);
        glGetFloatv(GL_MODELVIEW_MATRIX, @data);
        glPopMatrix;
    end;

    procedure Matrix.makeIdentity;
    begin
        glMATRIXMODE(GL_MODELVIEW);
        glPushMatrix;
        glLoadIdentity;
        glGetFloatv(GL_MODELVIEW_MATRIX, @data);
        glPopMatrix;
    end;

    procedure Matrix.useThis;
    begin
        glMATRIXMODE(GL_MODELVIEW);
        glLoadMatrixf(@data);
    end;

    procedure Matrix.setData(dataIn : M4);
    var i,j : integer;
    begin
      for i:=0 to 3 do begin
        for j:=0 to 3 do begin
          data[i*4+j]:=dataIn[i,j];
        end;
      end;
    end;

    function Matrix.getData : M4;
    var i,j : integer;
        resul : M4;
    begin
      for i:=0 to 3 do begin
        for j:=0 to 3 do begin
          resul[i,j]:=data[i*4+j];
        end;
      end;

      getData:=resul;
    end;

    function multiplyMatrixWithVec(matr : M4;vec : V3) : V3;
    var tmp : V4;
        resu : V3;
    begin
      tmp[0]:=matr[0,0]*vec[0]+matr[1,0]*vec[1]+matr[2,0]*vec[2]+matr[3,0]*1.0;
      tmp[1]:=matr[0,1]*vec[0]+matr[1,1]*vec[1]+matr[2,1]*vec[2]+matr[3,1]*1.0;
      tmp[2]:=matr[0,2]*vec[0]+matr[1,2]*vec[1]+matr[2,2]*vec[2]+matr[3,2]*1.0;
      tmp[3]:=matr[0,3]*vec[0]+matr[1,3]*vec[1]+matr[2,3]*vec[2]+matr[3,3]*1.0;

      resu[0]:=tmp[0]*tmp[3];
      resu[1]:=tmp[1]*tmp[3];
      resu[2]:=tmp[2]*tmp[3];

      multiplyMatrixWithVec:=resu;
    end;

    function vec3(x:GLFloat;y:GLFloat;z:GLFloat) : V3;
    var buffer : V3;
    begin
      buffer[0]:=x;
      buffer[1]:=y;
      buffer[2]:=z;

      vec3:=buffer;
    end;
end.
 