unit Car;

interface

uses Matrices, Materials, Polygon, NewtonImport, game, Textures, SysUtils, objload;


//------------------------------------------------------------------------------

type  tCar = class
    //Fizika
    Tire      : array[0..3] of integer;
    Joint     : PNewtonJoint;
    Body      : PNewtonBody;
    //Vizulis
    graphBody, graphWheel      : tMesh;
    textNum, wheelTextNum   : cardinal;
    material, tireMaterial  : tTexturedMaterial;
    graphBodyMatrix : Matrix;
    tireMat   : array[0..3] of Matrix;

    procedure draw;
    constructor Create(pMatrix : M4; cName : string);
    destructor Destroy;
end;

implementation

procedure PSetTransform(const Body : PNewtonBody; const Matrixx : PFloat); cdecl;
var
 TmpVehicle : tCar;
 tmpM4      : M4;
 i, j       : integer;
 point      : PFloat;
begin
  TmpVehicle := tCar(NewtonBodyGetUserData(Body));
  point:=Matrixx;
  //NewtonBodyGetMatrix(tmpVehicle.Body,@tmpM4[0,0]);
  for i:=0 to 3 do
  begin
     for j:=0 to 3 do
     begin
        point:=Matrixx;
        inc(point,i*4+j);
        tmpM4[i,j]:=point^;
     end;
  end;
  TmpVehicle.graphBodyMatrix.setData(tmpM4);
end;

procedure PTireUpdate(const Vehicle : PNewtonJoint); cdecl;
var
  tmpCar : tCar;
  tireId : integer;
  tmp    : M4;
  Accel  : Single;
begin
  tireId:=Integer(NewtonVehicleGetFirstTireID(Vehicle));
  tmpCar:=tCar(NewtonVehicleGetTireUserData(vehicle,pointer(tireId)));
  if(carctr.Accelerate) then begin
    if (carctr.SteerLeft) then
      NewtonVehicleSetTireTorque(Vehicle,pointer(tmpCar.Tire[3]),200)
    else if (carctr.SteerRight) then
      NewtonVehicleSetTireTorque(Vehicle,pointer(tmpCar.Tire[2]),200)
    else begin
      carctr.GearDown:=false;
      NewtonVehicleSetTireTorque(Vehicle,pointer(tmpCar.Tire[2]),400);
      NewtonVehicleSetTireTorque(Vehicle,pointer(tmpCar.Tire[3]),400);
    end;
  end;
  if(carctr.GearDown) then begin
      NewtonVehicleSetTireTorque(Vehicle,pointer(tmpCar.Tire[2]),-100);
      NewtonVehicleSetTireTorque(Vehicle,pointer(tmpCar.Tire[3]),-100);
    end;
  if(carctr.Brake) then begin
    Accel := NewtonVehicleTireCalculateMaxBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[0]));
    NewtonVehicleTireSetBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[0]), Accel, 225);

    Accel := NewtonVehicleTireCalculateMaxBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[1]));
    NewtonVehicleTireSetBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[1]), Accel, 225);

    Accel := NewtonVehicleTireCalculateMaxBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[2]));
    NewtonVehicleTireSetBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[2]), Accel, 225);

    Accel := NewtonVehicleTireCalculateMaxBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[3]));
    NewtonVehicleTireSetBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[3]), Accel, 225);
  end;

  if (carctr.Handbrake) then begin
    Accel := NewtonVehicleTireCalculateMaxBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[2]));
    NewtonVehicleTireSetBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[2]), Accel, 225);

    Accel := NewtonVehicleTireCalculateMaxBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[3]));
    NewtonVehicleTireSetBrakeAcceleration(Vehicle, Pointer(tmpCar.Tire[3]), Accel, 225);
  end;

  if(carctr.SteerLeft) then begin
    if carctr.Bigturn then begin
      NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[0]),0.4);
      NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[1]),0.4);
    end
    else begin
    NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[0]),0.2);
    NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[1]),0.2);
    end;
  end
  else if(carctr.SteerRight) then begin
    if carctr.Bigturn then begin
      NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[0]),-0.4);
      NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[1]),-0.4);
    end
    else begin
    NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[0]),-0.2);
    NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[1]),-0.2);
    end;
  end
  else
  begin
    NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[0]),0);
    NewtonVehicleSetTireSteerAngle(Vehicle,pointer(tmpCar.Tire[1]),0);
  end;

  NewtonVehicleGetTireMatrix(Vehicle,pointer(tmpCar.Tire[0]),@tmp[0,0]);
  tmpCar.tireMat[0].setData(tmp);
  NewtonVehicleGetTireMatrix(Vehicle,pointer(tmpCar.Tire[1]),@tmp[0,0]);
  tmpCar.tireMat[1].setData(tmp);
  NewtonVehicleGetTireMatrix(Vehicle,pointer(tmpCar.Tire[2]),@tmp[0,0]);
  tmpCar.tireMat[2].setData(tmp);
  NewtonVehicleGetTireMatrix(Vehicle,pointer(tmpCar.Tire[3]),@tmp[0,0]);
  tmpCar.tireMat[3].setData(tmp);
end;

procedure PhysicsApplyGravityForce(const body : PNewtonBody); cdecl;
var
 Mass    : Single;
 Inertia : V3;
 Force   : V3;
 matr    : M4;
begin
  NewtonBodyGetMassMatrix(Body, @Mass, @Inertia[0], @Inertia[1], @Inertia[2]);
  Force:=vec3(0, -9.8 * Mass,  0.0);
  NewtonBodyAddForce(Body, @Force[0]);
end;

constructor tCar.Create(pMatrix : M4; cName : string);
var
 Mass     : Single;
 Ixx      : Single;
 Iyy      : Single;
 Izz      : Single;
 Collider : PNewtonCollision;
 UpDir    : V3;
 BoxSize  : V3;

 x,y,z    : float;
 tireWidth, tireRadius : float;
 tireMass             : Single;
 tireSuspensionShock  : Single;
 tireSuspensionSpring : Single;
 tireSuspensionLength : Single;
 tmpTireMat  : M4;
 pin      : V3;

 dir : string;
 f   : text;
begin
  //Vizulis dolgok
  dir:=getCurrentDir;
  loadTexture(dir+'/cars/'+cName+'/cartext.tga',textNum,false);
  loadTexture(dir+'/cars/'+cName+'/wheel.tga',wheelTextNum,false);
  graphBody:=loadObjFile('cars/'+cName+'/car.obj');
  graphWheel:=loadObjFile('cars/'+cName+'/wheel.obj');
  material:=tTexturedMaterial.Create;
  material.setTexture(textNum);
  tireMaterial:=tTexturedMaterial.Create;
  tireMaterial.setTexture(wheelTextNum);
  graphBodyMatrix:=Matrix.Create;

  assignFile(f, 'cars/'+cName+'/car.ini');
  reset(f);
  readln(f,dir);
  readln(f,dir);
  Mass:=StrToFloat(dir)/4;
  readln(f,dir);
  BoxSize[0]:=StrToFloat(dir);
  readln(f,dir);
  BoxSize[1]:=StrToFloat(dir);
  readln(f,dir);
  BoxSize[2]:=StrToFloat(dir);

  Collider := NewtonCreateBox(NewtonWorld, BoxSize[0],BoxSize[1],BoxSize[2], nil);
  Body     := NewtonCreateBody(NewtonWorld, Collider);

  Ixx := Mass * (1.3 * BoxSize[1] * BoxSize[1] + BoxSize[2] * BoxSize[2]) / 12;
  Iyy := Mass * (BoxSize[0] * BoxSize[0] + BoxSize[2] * BoxSize[2]) / 12;
  Izz := Mass * (BoxSize[0] * BoxSize[0] + 1.3 * BoxSize[1] * BoxSize[1]) / 12;

  NewtonReleaseCollision(NewtonWorld, Collider);
  // Set transform callback
  NewtonBodySetTransformCallback(Body, PSetTransform);
  // Set the force and torque call back funtion
  NewtonBodySetForceAndTorqueCallback(Body, PhysicsApplyGravityForce);
  // Set mass matrix
  NewtonBodySetMassMatrix(Body, Mass, Ixx, Iyy, Izz);
  // Set the matrix for the rigid body
  NewtonBodySetMatrix(Body, @pMatrix[0,0]);
  NewtonBodySetUserData(Body, self);
  NewtonBodySetAutoFreeze(Body, 0);
  //Vehicle Joint
  UpDir := Vec3(pMatrix[0,1], pMatrix[1,1], pMatrix[2,1]);
  Joint := NewtonConstraintCreateVehicle(NewtonWorld, @UpDir[0], Body);
  NewtonVehicleSetTireCallback(Joint, @PTireUpdate);
  //Kerekek hozzadsa
  readln(f,dir);
  tireWidth:=StrToFloat(dir);
  readln(f,dir);
  tireRadius:=StrToFloat(dir);
  tireMass             := Mass/20;
  tireSuspensionLength := 0.3;
  tireSuspensionSpring := (200 * 9.8) / 0.3;
  tireSuspensionShock  := 2 * Sqrt(tireSuspensionSpring);
  //Bal els
  tireMat[0]:=Matrix.Create;
  tireMat[0].makeIdentity;
  readln(f,dir);
  x:=StrToFloat(dir);
  readln(f,dir);
  y:=StrToFloat(dir);
  readln(f,dir);
  z:=StrToFloat(dir);
  tireMat[0].translate(x,y,z);
  tmpTireMat:=tireMat[0].getData;
  pin:=Vec3(0,0,1);
  tire[0]:=Integer(NewtonVehicleAddTire(Joint,@tmpTireMat[0,0],@pin[0],tireMass,tireWidth,
                          tireRadius,tireSuspensionShock,tireSuspensionSpring,tireSuspensionLength,
                          self,0));
  NewtonVehicleSetTireMaxLongitudinalSlideSpeed(Joint, Pointer(tire[0]), 300);
  //Jobb els
  tireMat[1]:=Matrix.Create;
  tireMat[1].makeIdentity;
  readln(f,dir);
  x:=StrToFloat(dir);
  readln(f,dir);
  y:=StrToFloat(dir);
  readln(f,dir);
  z:=StrToFloat(dir);
  tireMat[1].translate(x,y,z);
  tmpTireMat:=tireMat[1].getData;
  pin:=Vec3(0,0,1);
  tire[1]:=Integer(NewtonVehicleAddTire(Joint,@tmpTireMat[0,0],@pin[0],tireMass,tireWidth,
                          tireRadius,tireSuspensionShock,tireSuspensionSpring,tireSuspensionLength,
                          self,0));
  NewtonVehicleSetTireMaxLongitudinalSlideSpeed(Joint, Pointer(tire[1]), 300);
  //Bal hts
  tireMat[2]:=Matrix.Create;
  tireMat[2].makeIdentity;
  readln(f,dir);
  x:=StrToFloat(dir);
  readln(f,dir);
  y:=StrToFloat(dir);
  readln(f,dir);
  z:=StrToFloat(dir);
  tireMat[2].translate(x,y,z);
  tmpTireMat:=tireMat[2].getData;
  pin:=Vec3(0,0,1);
  tire[2]:=Integer(NewtonVehicleAddTire(Joint,@tmpTireMat[0,0],@pin[0],tireMass,tireWidth,
                          tireRadius,tireSuspensionShock,tireSuspensionSpring,tireSuspensionLength,
                          self,0));
  NewtonVehicleSetTireMaxLongitudinalSlideSpeed(Joint, Pointer(tire[2]), 100);
  //Jobb hts
  tireMat[3]:=Matrix.Create;
  tireMat[3].makeIdentity;
  readln(f,dir);
  x:=StrToFloat(dir);
  readln(f,dir);
  y:=StrToFloat(dir);
  readln(f,dir);
  z:=StrToFloat(dir);
  tireMat[3].translate(x,y,z);
  tmpTireMat:=tireMat[3].getData;
  pin:=Vec3(0,0,1);
  tire[3]:=Integer(NewtonVehicleAddTire(Joint,@tmpTireMat[0,0],@pin[0],tireMass,tireWidth,
                          tireRadius,tireSuspensionShock,tireSuspensionSpring,tireSuspensionLength,
                          self,0));
  NewtonVehicleSetTireMaxLongitudinalSlideSpeed(Joint, Pointer(tire[3]), 100);

  closeFile(f);
end;

procedure tCar.draw;
var matrixfg : M4;
begin
// graphBodyMatrix.makeIdentity;
  graphBodyMatrix.useThis;
  material.draw(graphBody);
  //Kerekek
  tireMat[0].useThis;
  tireMaterial.draw(graphWheel);
  tireMat[1].useThis;
  tireMaterial.draw(graphWheel);
  tireMat[2].useThis;
  tireMaterial.draw(graphWheel);
  tireMat[3].useThis;
  tireMaterial.draw(graphWheel);
end;

destructor tCar.Destroy;
begin
  cleanMesh(graphBody);
  cleanMesh(graphWheel);
  material.Destroy;
  tireMaterial.Destroy;
end;

end.
